#include "StdAfx.h"
#include "TsConverter.h"


CTsConverter::CTsConverter(IEventHandler *pEventHandler)
	: m_pEventHandler(pEventHandler)
	, m_FileReader(this)
	, m_TsPacketParser(this)
	, m_TsDescrambler(this)
	, m_ProgManager(this)
	, m_TsDemuxer(this)
	, m_VideoFile(this)
	, m_AudioFile(this)
	, m_AacConverter(this)
	, m_WaveWriter(this)
{

}

CTsConverter::~CTsConverter(void)
{
	CancelConvert();
}

const bool CTsConverter::ConvertTsFile(LPCTSTR lpszTsFile, LPCTSTR lpszVideoFile, LPCTSTR lpszAudioFile, const bool bDecodeAac, const bool bLipSync)
{
/* 
	|AACfR[hȂ|
	m_FileReader  [m_TsPacketParser]  m_TsDescrambler  m_ProgManager  m_TsDemuxer  m_VideoFile
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ m_AudioFile
	|AACfR[h|
	m_FileReader  [m_TsPacketParser]  m_TsDescrambler  m_ProgManager  m_TsDemuxer  m_VideoFile
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ m_AacConverter  m_WaveWriter
*/

	// fR[_UN[Y
	CloseDecoders();

	try{
		// TSt@CJ
		if(!m_FileReader.OpenFile(lpszTsFile))throw 0UL;

		// fo̓t@C̎w肪ꍇ͊J
		if(lpszVideoFile){
			if(!m_VideoFile.OpenFile(lpszVideoFile))throw 1UL;
			m_TsDemuxer.SetOutputDecoder(&m_VideoFile, CTsDemuxer::OUTPUT_VIDEO);
			}

		// o̓t@C̎w肪ꍇ͊J
		if(lpszAudioFile){
			if(bDecodeAac){
				// AACfR[h
				if(!m_WaveWriter.OpenFile(lpszAudioFile))throw 2UL;
				m_TsDemuxer.SetOutputDecoder(&m_AacConverter, CTsDemuxer::OUTPUT_AUDIO);
				m_AacConverter.SetOutputDecoder(&m_WaveWriter);
				}
			else{
				// AACfR[hȂ
				if(!m_AudioFile.OpenFile(lpszAudioFile))throw 3UL;
				m_TsDemuxer.SetOutputDecoder(&m_AudioFile, CTsDemuxer::OUTPUT_AUDIO);
				}
			}

		// B-CASJ[hp\ȂƂTSfXNuڑ
		if(m_TsDescrambler.OpenBcasCard()){
			// B-CASJ[hp\
			m_TsPacketParser.SetOutputDecoder(&m_TsDescrambler);
			m_TsDescrambler.SetOutputDecoder(&m_ProgManager);	
			}
		else{
			// B-CASJ[hps
			m_TsPacketParser.SetOutputDecoder(&m_ProgManager);
			}

		// fR[_Otڑ
		m_FileReader.SetOutputDecoder(&m_TsPacketParser);
		m_ProgManager.SetOutputDecoder(&m_TsDemuxer);
		m_TsDemuxer.EnableLipSync(bLipSync);
		}
	catch(DWORD dwErrorCode){
		// G[
		CloseDecoders();
		return false;
		}
	
	// fR[_OtZbg
	m_FileReader.Reset();
	
	// Ro[gJn
	return m_FileReader.StartReadAnsync();
}

void CTsConverter::CancelConvert(void)
{
	// Ro[g~
	m_FileReader.StopReadAnsync();
}

void CTsConverter::CloseDecoders(void)
{
	// fR[_
	m_TsDescrambler.CloseBcasCard();
	m_FileReader.CloseFile();
	m_VideoFile.CloseFile();
	m_AudioFile.CloseFile();
	m_WaveWriter.CloseFile();
}

const DWORD CTsConverter::OnDecoderEvent(CMediaDecoder *pDecoder, const DWORD dwEventID, PVOID pParam)
{
	// fR[_̃Cxg󂯎(b)
	if(pDecoder == &m_ProgManager){
		
		// vO}l[W̃Cxg
		switch(dwEventID){
			case CProgManager::EID_SERVICE_LIST_UPDATED : {
				// T[rX̍\ω
				WORD wVideoPID = 0xFFFF;
				WORD wAudioPID = 0xFFFF;
				m_ProgManager.GetServiceEsPID(&wVideoPID, &wAudioPID);
				m_TsDemuxer.SetVideoPID(wVideoPID);
				m_TsDemuxer.SetAudioPID(wAudioPID);
				m_pEventHandler->OnTsConverterServiceName(TEXT("T[`..."));			
				return 0UL;
				}
			
			case CProgManager::EID_SERVICE_INFO_UPDATED : {
				// T[rX񂪍XVꂽ
				TCHAR szServiceName[1024] = {TEXT("s")};
				m_ProgManager.GetServiceName(szServiceName);
				m_pEventHandler->OnTsConverterServiceName(szServiceName);			
				return 0UL;
				}
			}
		}
	else if(pDecoder == &m_FileReader){
		
		// t@C[_̃Cxg
		switch(dwEventID){
			case CFileReader::EID_READ_ASYNC_START :
				// 񓯊[hJn
				m_pEventHandler->OnTsConverterStart(m_FileReader.GetFileSize());
				return 0UL;
			
			case CFileReader::EID_READ_ASYNC_END :
				// 񓯊[hI
				CloseDecoders();
				m_pEventHandler->OnTsConverterEnd(m_FileReader.GetFileSize());
				return 0UL;
			
			case CFileReader::EID_READ_ASYNC_POSTREAD :
				// 񓯊[h
				m_pEventHandler->OnTsConverterProgress(m_FileReader.GetReadPos(), m_FileReader.GetFileSize());
				return 0UL;
			}		
		}

	return 0UL;
}
